@@ -1,3 +1,6 @@
+from __future__ import absolute_import
+from __future__ import unicode_literals
+import six
 from copy import deepcopy
 import unittest2
 from jsonobject import *
@@ -6,6 +9,7 @@ from jsonobject.exceptions import (
     DeleteNotAllowed,
     WrappingAttributeError,
 )
+from six.moves import map
 
 
 class Features(JsonObject):
@@ -35,11 +39,11 @@ class Person(Document):
     last_name = StringProperty()
     features = ObjectProperty(Features)
     favorite_numbers = ListProperty(int)
-    tags = ListProperty(unicode)
+    tags = ListProperty(six.text_type)
 
     @property
     def full_name(self):
-        return u'{self.first_name} {self.last_name}'.format(self=self)
+        return '{self.first_name} {self.last_name}'.format(self=self)
 
 
 class FamilyMember(Person):
@@ -85,7 +89,7 @@ class JsonObjectTestCase(unittest2.TestCase):
         data = self._danny_data()
         danny = FamilyMember.wrap(data)
         self.assertEqual(danny.doc_type, 'FamilyMember')
-        self.assertIsInstance(danny.doc_type, unicode)
+        self.assertIsInstance(danny.doc_type, six.text_type)
         self.assertEqual(danny.first_name, 'Danny')
         self.assertEqual(danny.last_name, 'Roberts')
         self.assertEqual(danny.brothers[0].full_name, 'Alex Roberts')
@@ -98,7 +102,7 @@ class JsonObjectTestCase(unittest2.TestCase):
         danny.brothers[1].first_name = 'Nick'
         self.assertEqual(danny.brothers[1].full_name, 'Nick Roberts')
 
-        brothers = [{
+        brothers_json = [{
             'first_name': 'Alex',
             'last_name': 'Roberts',
         }, {
@@ -106,15 +110,19 @@ class JsonObjectTestCase(unittest2.TestCase):
             'last_name': 'Roberts',
         }]
         with self.assertRaises(AssertionError):
-            danny.brothers = brothers
+            danny.brothers = brothers_json
 
-        brothers = map(FamilyMember.wrap, brothers)
+        brothers = list(map(FamilyMember.wrap, brothers_json))
         danny.brothers = brothers
 
         self.assertEqual(danny.brothers, brothers)
         self.assertTrue(isinstance(danny.brothers, JsonArray))
         self.assertEqual(danny.to_json(), data)
 
+        new_brothers = list(map(FamilyMember.wrap, brothers_json))
+        danny.brothers[2:3] = new_brothers
+        self.assertEqual(len(danny.brothers), 4)
+
         danny.features.hair = 'blond'
         self.assertEqual(danny.features.hair, 'blond')
         with self.assertRaises(BadValueError):
@@ -137,12 +145,20 @@ class JsonObjectTestCase(unittest2.TestCase):
         for error_type in (AttributeError, WrappingAttributeError):
             with self.assertRaises(error_type) as cm:
                 Person.wrap({'full_name': 'Danny Roberts'})
-            self.assertEqual(
-                unicode(cm.exception),
-                "can't set attribute corresponding to "
-                "'full_name' on a <class 'test.tests.Person'> "
-                "while wrapping {'full_name': 'Danny Roberts'}"
-            )
+            if six.PY2:
+                self.assertEqual(
+                    six.text_type(cm.exception),
+                    "can't set attribute corresponding to "
+                    "u'full_name' on a <class 'test.tests.Person'> "
+                    "while wrapping {u'full_name': u'Danny Roberts'}"
+                )
+            else:
+                self.assertEqual(
+                    six.text_type(cm.exception),
+                    "can't set attribute corresponding to "
+                    "'full_name' on a <class 'test.tests.Person'> "
+                    "while wrapping {'full_name': 'Danny Roberts'}"
+                )
 
     def test_pickle(self):
         import pickle
@@ -202,7 +218,7 @@ class JsonObjectTestCase(unittest2.TestCase):
         self.assertEqual(p.to_json(), {'a': None, 'b': json_end['b']})
         p['a'] = [1, 2, 3]
         self.assertEqual(p.to_json(), json_end)
-        self.assertEqual(p.keys(), p.to_json().keys())
+        self.assertEqual(list(p.keys()), list(p.to_json().keys()))
 
     def test_competing_names(self):
         with self.assertRaises(AssertionError):
@@ -369,7 +385,7 @@ class JsonObjectTestCase(unittest2.TestCase):
             l2 = ListProperty(IntegerProperty)
         d = Dummy()
         longint = 2 ** 63
-        self.assertIsInstance(longint, long)
+        self.assertIsInstance(longint, six.integer_types)
         d.i = longint
         self.assertEqual(d.i, longint)
         d.l = [longint]
@@ -383,7 +399,71 @@ class JsonObjectTestCase(unittest2.TestCase):
             string_list = ListProperty(StringProperty)
 
         foo = Foo({'string_list': ['a', 'b', 'c']})
-        self.assertEqual(foo.string_list, ['a', 'b', 'c'])
+        self.assertEqual(foo.string_list, ['a', 'b' , 'c'])
+
+    def test_typed_dict_of_dict(self):
+
+        class City(JsonObject):
+            _allow_dynamic_properties = False
+            name = StringProperty()
+
+        class Foo(JsonObject):
+            _allow_dynamic_properties = False
+            cities_by_state_by_country = DictProperty(DictProperty(City))
+
+        # testing an internal assumption; can remove if internals change
+        self.assertEqual(Foo.cities_by_state_by_country.item_wrapper.item_wrapper.item_type, City)
+
+        city = City.wrap({'name': 'Boston'})
+        with self.assertRaises(AttributeError):
+            city.off_spec = 'bar'
+
+        foo = Foo.wrap({'cities_by_state_by_country': {'USA': {'MA': {'name': 'Boston'}}}})
+        self.assertIsInstance(foo.cities_by_state_by_country['USA']['MA'], City)
+        with self.assertRaises(AttributeError):
+            foo.cities_by_state_by_country['USA']['MA'].off_spec = 'bar'
+
+    def test_object_property_with_lambda(self):
+        class Bar(JsonObject):
+            string = StringProperty()
+
+        class Foo(JsonObject):
+            bar = ObjectProperty(lambda: Bar)
+
+        foo = Foo()
+        self.assertIsInstance(foo.bar, Bar)
+
+
+class PropertyInsideContainerTest(unittest2.TestCase):
+
+    def test_default_is_required(self):
+        class Foo(JsonObject):
+            container = ListProperty(int)
+
+        with self.assertRaises(BadValueError):
+            Foo(container=[None])
+
+    def test_property_class_required(self):
+        class Foo(JsonObject):
+            container = ListProperty(IntegerProperty)
+
+        with self.assertRaises(BadValueError):
+            Foo(container=[None])
+
+    def test_property(self):
+        class Foo(JsonObject):
+            container = ListProperty(IntegerProperty())
+
+        # assert does not error
+        Foo(container=[None])
+
+    def test_required_property(self):
+
+        class Foo(JsonObject):
+            container = ListProperty(IntegerProperty(required=True))
+
+        with self.assertRaises(BadValueError):
+            Foo(container=[None])
 
 
 class LazyValidationTest(unittest2.TestCase):
@@ -439,6 +519,47 @@ class LazyValidationTest(unittest2.TestCase):
         foo.bars[0].string = 'hi'
         self._validate_not_raises(foo)
 
+    def test_list_update_by_index(self):
+        class Foo(JsonObject):
+            bar = ListProperty()
+
+        foo = Foo()
+        foo.bar.append(1)
+        self.assertEqual(foo.bar, foo.to_json()['bar'])
+        foo.bar[0] = 2
+        self.assertEqual(foo.bar, foo.to_json()['bar'])
+        self.assertEqual(2, foo.to_json()['bar'][0])
+
+    def test_schema_list_update_by_index(self):
+        class Bar(JsonObject):
+            string = StringProperty()
+
+        class Foo(JsonObject):
+            bar = ListProperty(Bar)
+
+        foo = Foo()
+        foo.bar.append(Bar(string='hi'))
+        self.assertEqual(foo.bar[0].string, foo.to_json()['bar'][0]['string'])
+        self.assertIsInstance(foo.bar[0], Bar)
+        self.assertIsInstance(foo.to_json()['bar'][0], dict)
+        foo.bar[0] = Bar(string='lo')
+        self.assertEqual(foo.bar[0].string, foo.to_json()['bar'][0]['string'])
+        self.assertEqual('lo', foo.to_json()['bar'][0]['string'])
+        self.assertIsInstance(foo.bar[0], Bar)
+        self.assertIsInstance(foo.to_json()['bar'][0], dict)
+
+    def test_list_plus_equals(self):
+        class Foo(JsonObject):
+            bar = ListProperty()
+
+        foo = Foo()
+        foo.bar = [1, 2, 3]
+        self.assertEqual(foo.bar, [1, 2, 3])
+        self.assertEqual(foo.to_json()['bar'], [1, 2, 3])
+        foo.bar += [4]
+        self.assertEqual(foo.bar, [1, 2, 3, 4])
+        self.assertEqual(foo.to_json()['bar'], [1, 2, 3, 4])
+
     def test_dict(self):
         class Bar(JsonObject):
             _validate_required_lazily = True
@@ -503,15 +624,15 @@ class PropertyTestCase(unittest2.TestCase):
         self.assertEqual(foo.to_json()['decimal'], '2.0')
 
         foo.decimal = 3
-        self.assertEqual(foo.decimal, decimal.Decimal(3L))
+        self.assertEqual(foo.decimal, decimal.Decimal(3))
         self.assertEqual(foo.to_json()['decimal'], '3')
 
-        foo.decimal = 4L
-        self.assertEqual(foo.decimal, decimal.Decimal(4L))
+        foo.decimal = 4
+        self.assertEqual(foo.decimal, decimal.Decimal(4))
         self.assertEqual(foo.to_json()['decimal'], '4')
 
         foo.decimal = 5.25
-        self.assertEqual(foo.decimal, decimal.Decimal(unicode(5.25)))
+        self.assertEqual(foo.decimal, decimal.Decimal('5.25'))
         self.assertEqual(foo.to_json()['decimal'], '5.25')
 
     def test_dict(self):
@@ -645,7 +766,7 @@ class DynamicConversionTestCase(unittest2.TestCase):
     class Foo(JsonObject):
         pass
     string_date = '2012-01-01'
-    date_date = datetime.date(2012, 01, 01)
+    date_date = datetime.date(2012, 1, 1)
 
     def _test_dynamic_conversion(self, foo):
         string_date = self.string_date
@@ -719,7 +840,7 @@ class User(JsonObject):
     name = StringProperty()
     active = BooleanProperty(default=False, required=True)
     date_joined = DateTimeProperty()
-    tags = ListProperty(unicode)
+    tags = ListProperty(six.text_type)
 
 
 class TestExactDateTime(unittest2.TestCase):
